# in your SearchWorker.run()
import logging, traceback
from PyQt6.QtCore import pyqtSignal, QThread

class SearchWorker(QThread):
    log = pyqtSignal(str)
    done = pyqtSignal(list)

    def __init__(self, params):
        super().__init__()
        self.params = params

    def run(self):
        try:
            logging.info("Search started: %s", self.params)
            # ... your search logic here, emit self.log(...) as you go ...
            results = self._do_search(self.params)
            self.done.emit(results or [])
            logging.info("Search finished: %d hits", len(results or []))
        except Exception as e:
            tb = "".join(traceback.format_exc())
            logging.exception("Worker crashed: %s", e)
            self.log.emit("❌ Worker crashed:\n" + tb)
            # Do NOT re-raise — keep the app alive



#!/usr/bin/env python3
import clipboard,os, logging, traceback, threading, io, sys, faulthandler, traceback, signal
from typing import *
from dataclasses import dataclass
from logging.handlers import RotatingFileHandler
from abstract_utilities.dynimport import import_symbols_to_parent
import_modules = [
    {"module":'abstract_paths.content_utils.file_utils',"symbols":['get_directory_map','findGlobFiles']},
    {"module":'abstract_paths.file_filtering.file_filters',"symbols":['collect_filepaths']},
    {"module":'abstract_paths.python_utils.utils.utils',"symbols":['get_py_script_paths']},
    {"module":'abstract_paths.content_utils.diff_engine',"symbols":['plan_previews','apply_diff_text','ApplyReport','write_text_atomic']},
    {"module":'abstract_paths.content_utils.find_content',"symbols":['findContent','getLineNums','findContentAndEdit','findContent','get_line_content']},
    {"module":'abstract_paths.content_utils.find_content',"symbols":['getLineNums','findContentAndEdit','findContent','get_line_content']}
     ]
import_symbols_to_parent(import_modules, update_all=True)
from PyQt6 import *







from PyQt6 import QtCore
from PyQt6.QtWidgets import (
    QSizePolicy, QWidget, QLayout, QToolButton
    )
from PyQt6.QtGui import QKeySequence,QShortcut
from PyQt6.QtCore import QPropertyAnimation

def set_visible(self, name: str, visible: bool):
    sec = self._sections.get(name)
    if not sec:
        return
    btn: QToolButton = sec["button"]
    if btn.isChecked() != visible:
        btn.setChecked(visible)  # this triggers toggle handler

def is_visible(self, name: str) -> bool:
    sec = self._sections.get(name)
    return bool(sec and sec["button"].isChecked())

def button(self, name: str) -> QToolButton | None:
    sec = self._sections.get(name)
    return sec["button"] if sec else None

def container(self, name: str) -> QWidget | None:
    sec = self._sections.get(name)
    return sec["container"] if sec else None
# ---- public API ----
_QT_MAX = 16777215  # QWIDGETSIZE_MAX

def register(self, *,
             name: str,
             container: QWidget | QLayout,
             button: QToolButton | None = None,
             start_visible: bool | None = None,
             animate: bool | None = None,
             shortcut: str | None = None,
             button_host_layout: QLayout | None = None,
             button_text_open: str = "−",
             button_text_closed: str = "+",
             persist: bool = True) -> QToolButton:
    """
    Flicker-free collapsible section:
    - Keep container visible; animate maximumHeight 0 <-> QWIDGETSIZE_MAX.
    - No owner.adjustSize() on toggle.
    - No sizeHint() measurement dependency.
    """
    # Wrap layouts
    if isinstance(container, QLayout):
        container = wrap_layout(container)

    # Button
    if button is None:
        button = QToolButton(self._owner)
        button.setCheckable(True)
        button.setAutoRaise(True)
        button.setToolTip(f"Toggle {name}")
        if button_host_layout is not None:
            button_host_layout.addWidget(button)

    key  = f"section/{name}/visible"
    anim = self._animate_default if animate is None else animate

    # Restore persisted state
    if start_visible is None:
        vis = self._settings.value(key, True, type=bool) if persist else True
    else:
        vis = bool(start_visible)

    # Always visible; we collapse by maxHeight=0
    container.setVisible(True)
    container.setMinimumHeight(0)
    container.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)

    # Initial height
    container.setMaximumHeight(_QT_MAX if vis else 0)

    animator = None
    if anim:
        animator = QPropertyAnimation(container, b"maximumHeight", self)
        animator.setDuration(self._anim_ms)
        animator.setEasingCurve(QtCore.QEasingCurve.Type.InOutCubic)

    # Button state/label
    button.setChecked(vis)
    button.setText(button_text_open if vis else button_text_closed)

    def _apply(checked: bool):
        button.setText(button_text_open if checked else button_text_closed)
        end_h = _QT_MAX if checked else 0

        if animator:
            start_h = container.maximumHeight()
            animator.stop()
            animator.setStartValue(start_h)
            animator.setEndValue(end_h)
            try:
                animator.finished.disconnect()
            except TypeError:
                pass
            animator.finished.connect(container.updateGeometry)
            animator.start()
        else:
            container.setMaximumHeight(end_h)
            container.updateGeometry()

        if persist:
            self._settings.setValue(key, checked)
        self.toggled.emit(name, checked)
        # DO NOT call self._owner.adjustSize() here — it causes flicker

    button.toggled.connect(_apply)

    if shortcut:
        QShortcut(QKeySequence(shortcut), self._owner, activated=lambda: button.toggle())

    self._sections[name] = dict(
        container=container, button=button, animator=animator,
        text_open=button_text_open, text_closed=button_text_closed,
        key=key, persist=persist, animate=anim
    )
    return button



from abstract_utilities import get_abstract_import
define_defaults = get_abstract_import(
    module='abstract_paths',
    symbol='define_defaults')
get_files_and_dirs = get_abstract_import(
    module='abstract_paths',
    symbol='get_files_and_dirs')





from .imports import *
from .initFuncs import initFuncs
# -----------------------------------------------------------------------------
#  supervised child process runner ------------------------------------------------------------
# -----------------------------------------------------------------------------
class launcherWindowTab(QtWidgets.QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Universal App Launcher (supervised)")
        self.resize(1000, 700)

        # central dummy (replace with your real UI)
        central = QtWidgets.QWidget(); self.setCentralWidget(central)
        v = QtWidgets.QVBoxLayout(central)

        # command entry + buttons
        row = QtWidgets.QHBoxLayout()
        self.cmd_edit = QtWidgets.QLineEdit()
        self.cmd_edit.setPlaceholderText("Command to run (e.g. python -u your_app.py or /usr/bin/someapp)")
        self.run_btn = QtWidgets.QPushButton("Run")
        self.stop_btn = QtWidgets.QPushButton("Stop")
        row.addWidget(self.cmd_edit); row.addWidget(self.run_btn); row.addWidget(self.stop_btn)
        v.addLayout(row)

        # toggleable bottom log pane
        self.log_pane = logPaneTab(self)
        self.log_pane.setVisible(True)  # start visible; you can default to False
        v.addWidget(self.log_pane)

        # runner
        self.runner = appRunnerTab(self.log_pane, autorestart=True, parent=self)

        # actions
        self.run_btn.clicked.connect(self._on_run)
        self.stop_btn.clicked.connect(self.runner.stop)

        # menu / toolbar toggle
        self.toggle_log_act = QtGui.QAction("Show/Hide Log", self, checkable=True, checked=True)
        self.toggle_log_act.triggered.connect(lambda checked: self.log_pane.setVisible(checked))
        tb = self.addToolBar("View"); tb.addAction(self.toggle_log_act)

        # hotkey: F12 toggles log
        QtGui.QShortcut(QtGui.QKeySequence("F12"), self, activated=lambda: self.toggle_log_act.trigger())


launcherWindowTab = initFuncs(launcherWindowTab)



#!/usr/bin/env python3
from typing import *
from functools import partial, lru_cache
from abstract_utilities import eatAll
from abstract_utilities.type_utils import MIME_TYPES
from PyQt6 import QtGui, QtCore, QtWidgets
from PyQt6.QtGui import *
from PyQt6.QtCore import *
from PyQt6.QtWidgets import *
import time,  pydot, inspect, threading, enum, sys, requests, subprocess, os, logging, traceback, re, json,clipboard, logging, traceback, threading, io, sys, faulthandler, traceback, signal
from abstract_utilities import is_number,make_list,safe_read_from_json,read_from_file,make_dirs,eatAll
from dataclasses import dataclass
from logging.handlers import RotatingFileHandler



# open_file_funcs.py
import os, shutil, sys, subprocess
from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QListWidgetItem, QMessageBox, QWidget


def _extract_path_line_from_item(item: QListWidgetItem) -> tuple[str, int | None]:
    data = item.data(Qt.ItemDataRole.UserRole)
    print(f"from _extract_path_line_from_item data == {data}")
    if isinstance(data, dict) and "file_path" in data:
        return data["file_path"], data.get("line")
    text = item.text()
    path, sep, rest = text.partition(":")
    line = int(rest.strip()) if sep and rest.strip().isdigit() else None
    return path, line

def open_one(self, item: QListWidgetItem | None = None):
    print(f"from open_one item == {item}")
    # When called from code (no item), use current selection
    if item is None and hasattr(self, "list"):
        item = self.list.currentItem()

    if item is None:
        parent = self if isinstance(self, QWidget) else None
        QMessageBox.information(parent, "Open", "No item selected.")
        return

    path, line = _extract_path_line_from_item(item)
    open_in_editor(path, line)

def open_all_hits(self):
    if not hasattr(self, "list"):
        return
    for i in range(self.list.count()):
        itm = self.list.item(i)
        path, line = _extract_path_line_from_item(itm)
        open_in_editor(path, line)
# open_file_funcs.py
def open_in_editor(path: str, line: int | None = None):
    path = os.path.abspath(path)
    print(f"from open_in_editor path == {path}")
    if not os.path.exists(path):
        QMessageBox.warning(None, "Open File", f"File not found:\n{path}")
        return
    basename = os.path.basename(path)
    filename,ext = os.path.splitext(basename)
    if ext == 'py':
        subprocess.Popen([sys.executable, "-m", "idlelib", path])
        return
    # 1) explicit override, e.g. ABSTRACT_EDITOR="code -g {file}:{line}"
    tpl = os.environ.get("ABSTRACT_EDITOR")
    if tpl:
        target = tpl.format(file=path, line=(line or 1))
        subprocess.Popen(target.split())
        return

    # 2) VS Code first
    code = shutil.which("code")
    if code:
        target = f"{path}:{line or 1}"
        subprocess.Popen([code, "-g", target])
        return

    # 3) fallback: IDLE
    
